package eu.univento.lobby.story.events;

import eu.univento.core.api.entity.EntityModifier;
import eu.univento.core.api.entity.pathfinding.AStar;
import eu.univento.core.api.entity.pathfinding.PathingResult;
import eu.univento.core.api.entity.pathfinding.Tile;
import eu.univento.core.api.player.CustomPlayer;
import eu.univento.core.api.story.StoryMission;
import eu.univento.lobby.Lobby;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Giant;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.player.PlayerToggleSneakEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;

import java.util.ArrayList;
import java.util.HashSet;

/**
 * @author joethei
 * @version 0.1
 */
public class StartMissionEvents implements Listener{

    private StoryMission mission;

    public StartMissionEvents(StoryMission mission) {
        this.mission = mission;
        mission.registerEvent(this);
    }

    @EventHandler
    public void onSneak(PlayerToggleSneakEvent e) {
        CustomPlayer p = CustomPlayer.getPlayer(e.getPlayer());
        if(p == mission.getPlayer()) {
            if(p.isSneaking()) {
                p.sendMessage("Hallo du da");
                EntityModifier.Builder crepper = new EntityModifier(p.getWorld().spawnEntity(p.getLocation(), EntityType.CREEPER)).modify();
                crepper.setNoAI(true).followPlayer(p, 1.0F);

                Giant giant = p.getWorld().spawn(p.getLocation(), Giant.class);
                giant.getEquipment().setItemInMainHand(new ItemStack(Material.DIAMOND_SWORD));
                giant.addPotionEffect(new PotionEffect(PotionEffectType.INVISIBILITY, Integer.MAX_VALUE, 999));
                giant.setAI(false);
                giant.setGravity(false);

                runPathing(p.getLocation(), p.getTargetBlock(new HashSet<Byte>(), 50).getLocation(), 50);

                Bukkit.getScheduler().scheduleSyncDelayedTask(Lobby.getInstance(), () -> {
                    crepper.die();
                    giant.remove();
                }, 20 * 20L);
            }
        }
    }

    @EventHandler
    public void onDamage(EntityDamageEvent e) {
        if(e.getEntity() instanceof Giant) e.setCancelled(true);
    }

    private void runPathing(final Location start, final Location end, final int range){
        try {
            //create our pathfinder
            AStar path = new AStar(start, end, range);
            //get the list of nodes to walk to as a Tile object
            ArrayList<Tile> route = path.iterate();
            //get the result of the path trace
            PathingResult result = path.getPathingResult();

            switch(result){
                case SUCCESS :
                    //Path was successful. Do something here.
                    changePathBlocksToDiamond(start, route);
                    break;
                case NO_PATH :
                    //No path found, throw error.
                    System.out.println("No path found!");
                    break;
            }
        } catch (AStar.InvalidPathException e) {
            //InvalidPathException will be thrown if start or end block is air
            if(e.isStartNotSolid()){
                System.out.println("End block is not walkable");
            }
            if(e.isStartNotSolid()){
                System.out.println("Start block is not walkable");
            }
        }
    }

    private void changePathBlocksToDiamond(Location start, ArrayList<Tile> tiles){
        for(Tile t : tiles)
            t.getLocation(start).getBlock().setType(Material.DIAMOND_BLOCK);
    }
}